

/*--------------------------------------------------------------
 Программа:    Arduino Home Server 
 Автор:       С.С. Гранкин, http://www.factoblog.ru
 --------------------------------------------------------------*/

#include <SPI.h>
#include <Ethernet.h>
#include "DHT.h"
#include <LiquidCrystal.h>
#include <IRremote.h>


//Константы
#define DHTPIN 2
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE);

LiquidCrystal lcd(8, 9, 4, 5, 6, 7 );

byte mac[] = { 
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED }; // Мак адрес

EthernetClient client;

//Переменные
long unsigned int lowIn;                  // Время, в которое был принят сигнал отсутствия движения(LOW)    
boolean lockLow = true;                   // Флаг. false = значит движение уже обнаружено, true - уже известно, что движения нет
boolean takeLowTime;                      // Флаг. Сигнализирует о необходимости запомнить время начала отсутствия движения
boolean PIR = false;             // Вкл./Выкл. датчик движения
unsigned long int timeConn = millis();    // Частота отправки данных о t/h на сервер
unsigned long int sendSens = millis();    // Частота отправки данных о датчиках на сервер
int h;                           // Значение температуры
int t;                           // Значение влажности
int codeLCD = 0;                          // Код для LCDisplay()
boolean REBOOT = true;                    // Перезагрузка (не менять!)
IRrecv irrecv(19);                        // Пин, к которому подключен приемник
decode_results results;
char server[] = "home.telwik.ru";
byte load[8] = {
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11111
};
byte gradus[8] = {
  0b00110,
  0b01001,
  0b01001,
  0b00110,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

/*--------------------------------------------------------------
 Вводим необходимые настройки
 --------------------------------------------------------------*/
void setup()
{
  //Пины
  pinMode(3, OUTPUT);            // Подсветка LCD
  pinMode(14, OUTPUT);           // Реле-1
  pinMode(18, INPUT);            // Датчик движения

  //Значения
  digitalWrite(14, HIGH);         // Отключаем реле-1

  //Старт
  Serial.begin(9600);
  irrecv.enableIRIn();
  Ethernet.begin(mac);
  dht.begin();
  lcd.begin(16, 2);
  lcd.createChar(0,load);
  lcd.createChar(1,gradus);
  display("Calibrating PIR","sensor - 10 sec.");
  digitalWrite(3, HIGH);
  lcd.clear();
  for(int i = 0; i < 16; i++) {
    lcd.setCursor(0,0);
    lcd.print("Calibrating PIR");
    lcd.setCursor(i,2);
    lcd.write(byte(0));
    delay(1500);
  }
  digitalWrite(3, LOW);
  //Приветствие LCD
  display("Arduino Server","Status: is RUN");

}
/*--------------------------------------------------------------
 Основное тело программы
 --------------------------------------------------------------*/
void loop()
{
  h = dht.readHumidity();
  t = dht.readTemperature();
  motion(PIR);
  if (millis() - sendSens > 120000 || REBOOT==true) {
    sensor(PIR, REBOOT);
    sendSens = millis();
    display("CONNECT SERVER","Sending data");
  }
  if (millis() - timeConn > 14400000 || REBOOT == true) {
    sendData(t,h);
    timeConn = millis();
    REBOOT = false;
    display("CONNECT SERVER","Send temp/hum");
  }
  LCDispay(codeLCD);
  if (irrecv.decode(&results)) {
    switch (results.value) {
    case 16738455: // 1
      LCDispay(0);
      codeLCD = 0;
      break;

    case 16750695: // 2
      LCDispay(1);
      codeLCD = 1;
      break;

    case 16712445: //ОК
      digitalWrite(3, HIGH);
      delay(1500);
      digitalWrite(3, LOW);
      break;

    case 16728765: //*
      PIR = true;
      display("MOTION SENSOR","Status: ON");
      break;

    case 16732845: //#
      PIR = false;
      lockLow = true; 
      digitalWrite(14, HIGH);
      display("MOTION SENSOR","Status: OFF");
      break;
    }
    irrecv.resume(); 
  }

}
/*--------------------------------------------------------------
 Функция вывода информации на LCD KeyPad Shild
 Принемает параметр id.
 --------------------------------------------------------------*/
void LCDispay(int id) {
  switch(id) {
  case 1:
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Rele-1:OFF Rele-2:ON");
    lcd.setCursor(0,2);
    lcd.print("Light:1950");
    break;

  default:
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Temp:");
    lcd.setCursor(5,0);
    lcd.print(t);
    lcd.write(byte(1));
    lcd.setCursor(9,0);
    lcd.print("Hum:");
    lcd.setCursor(13,0);
    lcd.print(h);
    lcd.print("%");
    lcd.setCursor(0,2);
    lcd.print("Motion:");
    if (PIR == true) {
      lcd.print("ON");
      lcd.setCursor(7,2);
    } 
    else {
      lcd.print("OFF");
      lcd.setCursor(8,2);
    }
    break;
  }
}
/*--------------------------------------------------------------
 Функция отправляет данные о сотоянии датчиков, реле и тд.
 на WEB сервер.
 --------------------------------------------------------------*/
void sensor(int p1, int rb) {
  client.connect(server, 80);
  client.print( "GET /add.php?");
  client.print("k="); // Специальный код, например asREb25C
  client.print("&");
  client.print("p1=");
  client.print(p1);
  client.print("&");
  client.print("rb=");
  client.print(rb);
  client.print("&");
  client.print("ram=");
  client.print(freeRam());
  client.println(" HTTP/1.1");
  client.print( "Host: " );
  client.println(server);
  client.println( "Connection: close" );
  client.println();
  client.println();
  client.stop();
  client.flush();
}
/*--------------------------------------------------------------
 Функция датчика движения (PIR-sensor)
 --------------------------------------------------------------*/
void motion(int PIR) {

  if (PIR==true) {
    if(digitalRead(18) == HIGH) {
      if(lockLow) {
        lockLow = false;     
        digitalWrite(14, LOW);
        display("MOTION DETECTED","RELE-1 is ON");
        Serial.println(lowIn);
        Serial.println("MOTION DETECTED RELE-1 is ON");
      }        
      takeLowTime = true;
    } 
    else {      
      if(takeLowTime) {
        lowIn = millis();
        takeLowTime = false;
      }
      if(!lockLow && millis() - lowIn > 5000) { 
        lockLow = true;               
        digitalWrite(14, HIGH);
        display("MOTION FINISH","RELE-1 is OFF");
        Serial.println(lowIn);
        Serial.println("MOTION FINISH RELE-1 is OFF");
      }
    }
  }
}
/*--------------------------------------------------------------
 Функция отправляет данные о температуре и влажности на 
 WEB сервер.
 --------------------------------------------------------------*/
void sendData(float t, float h) {
  client.connect(server, 80);
  client.print( "GET /add.php?");
  client.print("k="); // Специальный код, например asREb25C
  client.print("&");
  client.print("t=");
  client.print(t);
  client.print("&");
  client.print("h=");
  client.print(h);
  client.println(" HTTP/1.1");
  client.print( "Host: " );
  client.println(server);
  client.println( "Connection: close" );
  client.println();
  client.println();
  client.stop();
  client.flush();
}
/*--------------------------------------------------------------
 Функция вывода информации на LCD KeyPad Shild
 Принемает параметр title1 - первая строка (max 16 символов),
 title2 - вторая сторока.
 --------------------------------------------------------------*/
void display(String title1, String title2) {
  lcd.clear();
  lcd.setCursor(0,0);
  digitalWrite(3, HIGH);
  lcd.print(title1);
  lcd.setCursor(0,2);
  lcd.print(title2);
  delay(1600);
  digitalWrite(3, LOW);
}
/*--------------------------------------------------------------
 Функция выводит количество свободных байт RAM
 --------------------------------------------------------------*/
int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}





